package de.firstdata.ipg.api.client.transaction;

import java.net.URL;
import java.util.Date;

import org.apache.commons.lang3.time.FastDateFormat;

import de.firstdata.ipg.api.client.transaction.object.IPGApiTransactionObject;
import de.firstdata.ipg.api.schema.Mandate;
import de.firstdata.ipg.api.schema.MandateType;
import de.firstdata.ipg.api.schema.SepaTxType;

/**
 * @author f2ba1qo
 */
public class SepaSaleTransactionObject extends IPGApiTransactionObject implements SepaSaleTransaction {

	public static final String SALE = "sale";

	private static final FastDateFormat DATE_FORMAT = FastDateFormat.getInstance("yyyyMMdd");

	public SepaSaleTransactionObject(final Amount amount, final String orderId) {
		super(amount, null, null, orderId);
		getSepaTxType().setType(SALE);
		final SepaData sepaData = new SepaData();
		setSepaData(sepaData);
		sepaData.setMandate(new Mandate());
	}

	/**
	 * Sepa Direct Debit has only sale type.
	 *
	 * @return "sale" constant
	 */
	@Override
	public String getType() {
		return SALE;
	}

	@Override
	public void setIban(final String iban) {
		this.sepaData.setIBAN(iban);
	}

	@Override
	public SepaTxType getSepaTxType() {
		if (this.sepaTxType == null) {
			this.sepaTxType = new SepaTxType();
		}
		return this.sepaTxType;
	}

	@Override
	public void setStoreId(final String storeId) {
		getSepaTxType().setStoreId(storeId);
	}

	@Override
	public void setMandateReference(final String mandateReference) {
		this.sepaData.getMandate().setReference(mandateReference);
	}

	@Override
	public void setMandateDate(final Date date) {
		this.sepaData.getMandate().setDate(convertDateToString(date));
	}

	@Override
	public void setMandateType(final MandateType mandateType) {
		this.sepaData.getMandate().setType(mandateType);
	}

	@Override
	public void setMandateUrl(final URL mandateUrl) {
		this.sepaData.getMandate().setUrl(String.valueOf(mandateUrl));
	}

	private String convertDateToString(final Date date) {
		return DATE_FORMAT.format(date);
	}

	@Override
	public String getStore() {
		return getSepaTxType().getStoreId();
	}
}
