package de.firstdata.ipg.api.client.transaction.object;

import java.util.Date;
import java.util.Locale;

import javax.xml.datatype.XMLGregorianCalendar;

import org.apache.commons.lang3.time.FastDateFormat;

import de.firstdata.ipg.api.client.transaction.Amount;
import de.firstdata.ipg.api.client.transaction.IPGApiTransaction;
import de.firstdata.ipg.api.schema.AdditionalRequestParameters;
import de.firstdata.ipg.api.schema.Billing;
import de.firstdata.ipg.api.schema.ClientLocale;
import de.firstdata.ipg.api.schema.GenderType;
import de.firstdata.ipg.api.schema.Payment;
import de.firstdata.ipg.api.schema.Shipping;
import de.firstdata.ipg.api.schema.TokenType;
import de.firstdata.ipg.api.schema.Transaction;
import de.firstdata.ipg.api.schema.TransactionDetails;

/**
 * <code>IPGApiTransaction</code> is the abstract super class of any other
 * transaction class. It contains methods for setting optional fields which are
 * common to all transactions regardless of their type.
 */
public abstract class IPGApiTransactionObject extends Transaction implements IPGApiTransaction {

	private static final FastDateFormat DATE_FORMATTER = FastDateFormat.getInstance("yyyyMMdd");

	protected static String getFormattedDate(final Date value) {
		return DATE_FORMATTER.format(value);
	}

	protected IPGApiTransactionObject(final Amount amount, final String hostedDataID, final String hostedDataStoreID, final String orderId) {
		if (orderId != null && orderId.length() > 0) {
			getTransactionDetails().setOrderId(orderId);
		}
		setPayment(amount);
		getPayment().getHostedDataID().add(hostedDataID);
		getPayment().setHostedDataStoreID(hostedDataStoreID);
	}

	@Override
	public void setComments(final String value) {
		getTransactionDetails().setComments(value);
	}

	/**
	 * sets the customer's IP address which can be used for fraud detection
	 *
	 * @param ip
	 *            the customer's IP address
	 */
	public void setIp(final String ip) {
		getTransactionDetails().setIp(ip);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.TransactionDetailsSetter#
	 * setOrderId (java.lang.String)
	 */
	public void setOrderId(final String orderId) {
		getTransactionDetails().setOrderId(orderId);
	}

	public void setTDate(final String tDate) {
		getTransactionDetails().setTDate(tDate);
	}

	@Override
	public void setMerchantTransactionId(final String merchantTransactionId) {
		getTransactionDetails().setMerchantTransactionId(merchantTransactionId);
	}

	@Override
	public void setReferencedMerchantTransactionId(final String referencedMerchantTransactionId) {
		getTransactionDetails().setReferencedMerchantTransactionId(referencedMerchantTransactionId);
	}

	@Override
	public void setIpgTransactionId(final Long ipgTransactionId) {
		getTransactionDetails().setIpgTransactionId(ipgTransactionId);
	}

	public void setInvoiceNumber(final String invoiceNumber) {
		getTransactionDetails().setInvoiceNumber(invoiceNumber);
	}

	public void setDynamicMerchantName(final String dynamicMerchantName) {
		getTransactionDetails().setDynamicMerchantName(dynamicMerchantName);
	}

	public void setPONumber(final String poNumber) {
		getTransactionDetails().setPONumber(poNumber);
	}

	@Override
	public void setClientLocale(final Locale value) {
		if (value == null) {
			this.clientLocale = null;
		} else {
			this.clientLocale = new ClientLocale();
			final String country = value.getCountry();
			if (country != null && country.length() > 0) {
				this.clientLocale.setCountry(country);
			}
			this.clientLocale.setLanguage(value.getLanguage());
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seede.firstdata.ipgapi.client.transaction.TransactionDetailsSetter#
	 * setTransactionOrigin(java.lang.String)
	 */
	public void setTransactionOrigin(final String transactionOrigin) {
		getTransactionDetails().setTransactionOrigin(transactionOrigin);
	}

	@Override
	public TransactionDetails getTransactionDetails() {
		if (this.transactionDetails == null) {
			this.transactionDetails = new TransactionDetails();
		}
		return this.transactionDetails;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setCustomerID
	 * (java.lang.String)
	 */
	public void setCustomerID(final String customerID) {
		getBilling().setCustomerID(customerID);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setName(java.
	 * lang.String)
	 */
	public void setName(final String name) {
		getBilling().setName(name);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setCompany(java
	 * .lang.String)
	 */
	public void setCompany(final String company) {
		getBilling().setCompany(company);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setAddress1(java
	 * .lang.String)
	 */
	public void setAddress1(final String address1) {
		getBilling().setAddress1(address1);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setAddress2(java
	 * .lang.String)
	 */
	public void setAddress2(final String address2) {
		getBilling().setAddress2(address2);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setCity(java.
	 * lang.String)
	 */
	public void setCity(final String city) {
		getBilling().setCity(city);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setState(java
	 * .lang.String)
	 */
	public void setState(final String state) {
		getBilling().setState(state);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setZip(java.lang
	 * .String)
	 */
	public void setZip(final String zip) {
		getBilling().setZip(zip);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setCountry(java
	 * .lang.String)
	 */
	public void setCountry(final String country) {
		getBilling().setCountry(country);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setPhone(java
	 * .lang.String)
	 */
	public void setPhone(final String phone) {
		getBilling().setPhone(phone);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setFax(java.lang
	 * .String)
	 */
	public void setFax(final String fax) {
		getBilling().setFax(fax);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setEmail(java
	 * .lang.String)
	 */
	public void setEmail(final String email) {
		getBilling().setEmail(email);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setPersonalNumber
	 * (java .lang.String)
	 */
	public void setPersonalNumber(final String personalNumber) {
		getBilling().setPersonalNumber(personalNumber);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setBirthDate
	 * (javax.xml.datatype.XMLGregorianCalendar)
	 */
	public void setBirthDate(final XMLGregorianCalendar birthDate) {
		getBilling().setBirthDate(birthDate);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setGender
	 * (de.firstdata.ipg.api.schema.GenderType)
	 */
	public void setGender(final GenderType gender) {
		getBilling().setGender(gender);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setFirstname
	 * (java .lang.String)
	 */
	public void setFirstname(final String firstname) {
		getBilling().setFirstname(firstname);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setSurname
	 * (java .lang.String)
	 */
	public void setSurname(final String surname) {
		getBilling().setSurname(surname);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setStreetName
	 * (java .lang.String)
	 */
	public void setStreetName(final String streetName) {
		getBilling().setStreetName(streetName);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setHouseNumber
	 * (java .lang.String)
	 */
	public void setHouseNumber(final String houseNumber) {
		getBilling().setHouseNumber(houseNumber);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setHouseExtension
	 * (java .lang.String)
	 */
	public void setHouseExtension(final String houseExtension) {
		getBilling().setHouseExtension(houseExtension);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setMobilePhone
	 * (java .lang.String)
	 */
	public void setMobilePhone(final String mobilePhone) {
		getBilling().setMobilePhone(mobilePhone);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.BillingSetter#setAddressNumber
	 * (java.lang.String)
	 */
	@Deprecated
	public void setAddressNumber(final String addressNumber) {
		getBilling().setAddrnum(addressNumber);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see
	 * de.firstdata.ipgapi.client.transaction.cc.ShippingSetter#setCarrier(java
	 * .lang.String)
	 */
	@Deprecated
	public void setType(final String type) {
		getShipping().setType(type);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seede.firstdata.ipgapi.client.transaction.cc.ShippingSetter#
	 * setShippingAddressName(java.lang.String)
	 */
	public void setShippingAddressName(final String name) {
		getShipping().setName(name);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.cc.ShippingSetter#
	 * setShippingAddress1 (java.lang.String)
	 */
	public void setShippingAddress1(final String address1) {
		getShipping().setAddress1(address1);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.firstdata.ipgapi.client.transaction.cc.ShippingSetter#
	 * setShippingAddress2 (java.lang.String)
	 */
	public void setShippingAddress2(final String address2) {
		getShipping().setAddress2(address2);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seede.firstdata.ipgapi.client.transaction.cc.ShippingSetter#
	 * setShippingAddressCity(java.lang.String)
	 */
	public void setShippingAddressCity(final String city) {
		getShipping().setCity(city);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seede.firstdata.ipgapi.client.transaction.cc.ShippingSetter#
	 * setShippingAddressState(java.lang.String)
	 */
	public void setShippingAddressState(final String state) {
		getShipping().setState(state);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seede.firstdata.ipgapi.client.transaction.cc.ShippingSetter#
	 * setShippingAddressZip(java.lang.String)
	 */
	public void setShippingAddressZip(final String zip) {
		getShipping().setZip(zip);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seede.firstdata.ipgapi.client.transaction.cc.ShippingSetter#
	 * setShippingAddressCountry(java.lang.String)
	 */
	public void setShippingAddressCountry(final String country) {
		getShipping().setCountry(country);
	}

	public void setDeclineHostedDataDuplicates(final boolean value) {
		getPayment().setDeclineHostedDataDuplicates(Boolean.valueOf(value));
	}

	public void setHostedDataID(final String value) {
		getPayment().getHostedDataID().add(value);
	}

	public void setHostedDataStoreID(final String value) {
		getPayment().setHostedDataStoreID(value);
	}

	public void setTokenType(final TokenType value) {
		getPayment().setTokenType(value);
	}

	public void setAssignToken(final Boolean value) {
		getPayment().setAssignToken(value);
	}

	public void setAdditionalRequestParameters(final AdditionalRequestParameters additionalRequestParameters) {
		getTransactionDetails().setAdditionalRequestParameters(additionalRequestParameters);
	}

	@Override
	public Billing getBilling() {
		if (this.billing == null) {
			this.billing = new Billing();
		}
		return this.billing;
	}

	@Override
	public Shipping getShipping() {
		if (this.shipping == null) {
			this.shipping = new Shipping();
		}
		return this.shipping;
	}

	@Override
	public Payment getPayment() {
		if (this.payment == null) {
			this.payment = new Payment();
		}
		return this.payment;
	}

	protected void setPayment(final Amount amount) {
		if (amount == null) {
			return;
		}
		getPayment();
		this.payment.setSubTotal(amount.subTotal);
		this.payment.setValueAddedTax(amount.valueAddedTax);
		this.payment.setLocalTax(amount.localTax);
		this.payment.setDeliveryAmount(amount.deliveryAmount);
		this.payment.setChargeTotal(amount.total);
		this.payment.setCurrency(amount.currency);
	}

	public void setMerchantAdviceCodeSupported(final Boolean merchantAdviceCodeSupported) {
		getTransactionDetails().setMerchantAdviceCodeSupported(merchantAdviceCodeSupported);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @seede.firstdata.ipgapi.client.transaction.TransactionDetailsSetter#
	 * setIpgDeferredAuth(java.lang.Boolean)
	 */
	public void setIpgDeferredAuth(final Boolean isDeferredAuth) {
		getTransactionDetails().setIpgDeferredAuth(isDeferredAuth);
	}
}
